/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;

namespace Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility {
	/// <summary>
	/// Constants are used to set parameter names in IDP and SP Sample Applications
	/// </summary>
	public class Constants {
		
		public const String USER_ID            = "userId";
		public const String USER_PASSWORD      = "UserPassword";
		public const String USER_INFO          = "userInfo";
		public const String USER_SESSION_ID    = "userSessionID";
		public const String SESSION_ID_PARAM   = "sessionID";

		public const String HOST_PF            = "hostPF";
		public const String HOST_APP_IDP       = "hostAppIdp";
		public const String HOST_APP_SP        = "hostAppSp";
		public const String ACCOUNT_LINKING    = "accountLinking";
		public const String IDP_DISCOVERY	   = "idpDiscovery";
		public const String PASSWORD		   = "password";
		public const String HOLDER_NAME	       = "holderName";
		
		public const String IDP_DIR            = "/idp";
		public const String SP_DIR             = "/sp";

		public const String START_SSO          = "/startSSO.ping";
		public const String START_SLO          = "/startSLO.ping";
		public const String SP_DEFEDERATE      = "/sp/defederate.ping";
		public const String TARGET_SSO         = "/";
		public const String TARGET_IDP_SLO     = "/";
		public const String TARGET_SP_SLO      = "/";
		
		public const String IDP_HOME_URL       = "/mainhandler.aspx";
		public const String SP_HOME_URL        = "/mainhandler.aspx";
		public const String IDP_SAMPLE_SSO     = "/SSOIdpHandler.aspx";
		public const String SP_SAMPLE_SSO      = "/SSOSpHandler.aspx";
		public const String IDP_SAMPLE_SLO     = "/SLOIdpHandler.aspx";
		public const String SP_SAMPLE_SLO      = "/SLOSpHandler.aspx";
		public const String SP_SAMPLE_LINKING  = "/AccountLinkingSpHandler.aspx";
		
		public const String PARTNER_IDP_ID     = "PartnerIdpId";
		public const String PARTNER_SP_ID      = "PartnerSpId";
		public const String SP_ADAPTER_ID      = "SpSessionAuthnAdapterId";

		public const String LOGIN_ASPX         = "/login.aspx";
		public const String ACCOUNT_LINK_PAGE  = "/accountLinking.aspx";
		public const String MAIN_PAGE          = "/mainForm.aspx";
		public const String ERROR_PAGE         = "/error.aspx?message=";
		public const String WEB_PREFIX         = "";
		public const String TARGET_RESOURCE    = "TargetResource";
		public const String IDP_ADAPTER_ID	   = "IdpAdapterId";
		public const String DESCRIPTION		   = "Description";
		public const String IDP_APP			   = "IdPApp";
		public const String IN_ERROR_RESOURCE  = "InErrorResource";
		public const String ERROR_MESSAGE      = "message";
		public const String RESUME_PATH        = "resumePath";

		public const String LOGIN_TYPE         = "loginType";
		public const String LOCAL_TYPE         = "local";
		public const String GLOBAL_TYPE        = "global";

		public const String PARAM_CMD          = "cmd";
		public const String PARAM_CMD_SSO      = "sso";
		public const String CMD_LOGOUT         = "logout";

		public const String TRANSFER_METHOD    = "transferMethod";
		public const String COOKIE             = "cookie";
		public const String QUERY              = "query";

		public const String FORCE_AUTH_PARAM   = "ForceAuthn";
		public const String IS_PASSIVE_PARAM   = "IsPassive";

		public const String TRUE_STRING_VALUE  = "true";

		public const String CONNECTION_NAME    = "ConnectionName";
		public const String CONNECTION_VALUE   = "ConnectionValue";
		
		public const String IDP_CONFIG_TABLE   = "idpConfigTable";
		public const String IDP_CONNECT_LIST   = "idpConnectionsList";
		public const String SP_CONFIG_TABLE    = "spConfigTable";
		public const String SP_CONNECT_LIST    = "spConnectionsList";

		public static String DisplayAuthnContext(String authnContext)
		{
			String DisplayName = null;
			switch(authnContext) 
			{
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:InternetProtocol" :
					DisplayName = "IP";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:InternetProtocolPassword" :
					DisplayName = "IP_PASSWORD";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:Kerberos" :
					DisplayName = "KERBEROS";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:MobileOneFactorUnregistered" :
					DisplayName = "MOBILE_ONE_FACTOR_UNREGISTERED";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:MobileTwoFactorUnregistered" :
					DisplayName = "MOBILE_TWO_FACTOR_UNREGISTERED";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:MobileOneFactorContract" :
					DisplayName = "MOBILE_ONE_FACTOR_CONTRACT";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:MobileTwoFactorContract" :
					DisplayName = "MOBILE_TWO_FACTOR_CONTRACT";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:Password" :
					DisplayName = "PASSWORD";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:PasswordProtectedTransport" :
					DisplayName = "PASSWORD_PROTECTED_TRANSPORT";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:PreviousSession" :
					DisplayName = "PREVIOUS_SESSION";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:X509" :
					DisplayName = "X509";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:PGP" :
					DisplayName = "PGP";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:SPKI" :
					DisplayName = "SPKI";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:XMLDSig" :
					DisplayName = "XMLDSIG";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:Smartcard" :
					DisplayName = "SMARTCARD";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:SmartcardPKI" :
					DisplayName = "SMARTCARD_PKI";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:SoftwarePKI" :
					DisplayName = "SOFTWARE_PKI";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:Telephony" :
					DisplayName = "TELEPHONY";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:NomadTelephony" :
					DisplayName = "NOMAD_TELEPHONY";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:PersonalTelephony" :
					DisplayName = "PERSONAL_TELEPHONY";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:AuthenticatedTelephony" :
					DisplayName = "AUTHENTICATED_TELEPHONY";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:SecureRemotePassword" :
					DisplayName = "SECURE_REMOTE_PASSWORD";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:TLSClient" :
					DisplayName = "TLS_CLIENT";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:TimeSyncToken" :
					DisplayName = "TIME_SYNC_TOKEN";
					break;
				case "urn:oasis:names:tc:SAML:2.0:ac:classes:unspecified" :
					DisplayName = "UNSPECIFIED";
					break;

				default :
					DisplayName = authnContext;
					break;
			}
			return DisplayName;
		}
	}
}