/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;
using System.Collections;
using System.Web;

namespace Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility {

	/// <summary>
	/// Links is used for creating url's to Pingfederate IdP and SP SSO and SLO
	/// services. Also it is used for creating url's to IdP and SP parts of
	/// Sample Application
	/// </summary>
	public class Links {

		public static String hostPF;
		private String hostPFIdp;
		private String hostPFSp;
		private String hostAppIdp;
		private String hostAppSp;

		public Links(Hashtable configTable, HttpRequest request) {

			if(configTable == null) {
				throw new NoSuchTagException("Invalid config-file");
			}

			hostPF     = ( (String)configTable[Constants.HOST_PF] ).Trim();

            Uri MyUrl = request.Url;

            this.hostAppIdp = MyUrl.Scheme + "://" + MyUrl.Host + ":" + MyUrl.Port + request.ApplicationPath;
			if(this.hostAppIdp != null) 
			{
				this.hostAppIdp.Trim();
			}
            this.hostAppSp = MyUrl.Scheme + "://" + MyUrl.Host + ":" + MyUrl.Port + request.ApplicationPath;
            if (this.hostAppSp != null)
            {
                this.hostAppSp.Trim();
            }
			if(this.hostAppSp != null) 
			{
				this.hostAppSp.Trim();
			}
			
			this.hostPFIdp  = hostPF + Constants.IDP_DIR;
			this.hostPFSp   = hostPF + Constants.SP_DIR;
		}

		public String getIdpHomeUrl() {
			//create url to IDP mainhandler.aspx
			String url = hostAppIdp;
			return url;
		}

		public String getSpHomeUrl() {
			//create url to SP mainhandler.aspx
			String url = hostAppSp;
			return url;
		}

		public String getSSOIdpHandlerUrl() {
			//create url to SSOIdpServlet for starting IdP initiated SSO
			String url = hostAppIdp + Constants.IDP_SAMPLE_SSO;
			return url;
		}

		public String getSSOSpHandlerUrl() {
			//create url to SSOIdpServlet for starting IdP initiated SSO
			String url = hostAppSp + Constants.SP_SAMPLE_SSO;
			return url;
		}

		public String getAccountLinkingSpHandlerUrl() {
			//create url to AccountLinkingSpHandler on the SP part of the Sample Application
			String url = hostAppSp + Constants.SP_SAMPLE_LINKING;
			return url;
		}

		public String getIdpStartSSOUrl(String partnerSpId) 
		{
			//create url to SSOIdpHandler for starting IdP initiated SSO
			String url = hostAppIdp + "/?" + Constants.PARAM_CMD + "=" + Constants.PARAM_CMD_SSO + "&resumePath=" + Constants.IDP_DIR + Constants.START_SSO + 
				"?" + Constants.PARTNER_SP_ID + "=" + partnerSpId; 
			return url;
		}

		public String getIdpStartSSOUrl(String partnerSpId, String TargetResource) 
		{
			//create url to SSOIdpHandler for starting IdP initiated SSO
			String url = hostAppIdp + "/?" + Constants.PARAM_CMD + "=" + Constants.PARAM_CMD_SSO + "&resumePath=" + Constants.IDP_DIR + Constants.START_SSO + 
				"?" + Constants.PARTNER_SP_ID + "=" + partnerSpId + 
				"&" + Constants.TARGET_RESOURCE + "=" + TargetResource;
			return url;
		}

		public String getIdpStartSSOUrl(String partnerSpId, String TargetResource, String IdpAdapterId) {
			//create url to SSOIdpHandler for starting IdP initiated SSO
			String url = hostAppIdp + "/?" + Constants.PARAM_CMD + "=" + Constants.PARAM_CMD_SSO + "&resumePath=" + Constants.IDP_DIR + Constants.START_SSO + 
				"?" + Constants.PARTNER_SP_ID + "=" + partnerSpId + 
				"&" + Constants.IDP_ADAPTER_ID + "=" + IdpAdapterId +
				"&" + Constants.TARGET_RESOURCE + "=" + TargetResource;
			return url;
		}

		public String getSpStartSSOUrl(String partnerIdpId,
			String forceAuthn, String isPassive) 
		{
			//create url to Pingfederate SP Start SSO service
			String url = hostPFSp + Constants.START_SSO + "?" + Constants.PARTNER_IDP_ID +
				"=" + partnerIdpId + "&" + Constants.TARGET_RESOURCE + "=" +
				hostAppSp + Constants.TARGET_SSO;

			//add ForceAuthn and IsPassive parameters if it is necessary
			String urlWithParams = addSSOParameters(url, forceAuthn, isPassive);
			return urlWithParams;
		}

		public String getSpStartSSOUrl(String partnerIdpId, String spSessionAuthnAdapterId,
				String forceAuthn, String isPassive) {
			//create url to Pingfederate SP Start SSO service
			String url = hostPFSp + Constants.START_SSO + "?" + Constants.PARTNER_IDP_ID +
				"=" + partnerIdpId + "&" + Constants.SP_ADAPTER_ID + "=" +
				spSessionAuthnAdapterId + "&" + Constants.TARGET_RESOURCE + "=" +
				hostAppSp + Constants.TARGET_SSO;

			//add ForceAuthn and IsPassive parameters if it is necessary
			String urlWithParams = addSSOParameters(url, forceAuthn, isPassive);
			return urlWithParams;
		}

		public String getIdpStartSLOUrl() {
			//create url to SLOIdpHandler for starting IdP initiated SLO
			String url = hostAppIdp + Constants.IDP_SAMPLE_SLO;
			return url;
		}

		public String getSpStartSLOUrl() {
			//create url to SLOSpHandler for starting SP initiated SLO
			String url = hostAppSp + Constants.SP_SAMPLE_SLO;
			return url;
		}

		public String getIdpSSOLink(String partnerSpId, String TargetResource) {
			//create url to Pingfederate IdP Start SSO service
			String url = hostPFIdp + Constants.START_SSO + "?" +
				Constants.PARTNER_SP_ID + "=" + partnerSpId + "&" +
				Constants.TARGET_RESOURCE + "=" + TargetResource; 
			return url;
		}

		public String getIdpSLOLink() {
			//create url to Pingfederate IdP Start SLO service
			String url = hostPFIdp + Constants.START_SLO + "?" +
				Constants.TARGET_RESOURCE + "=" + hostAppIdp + Constants.TARGET_IDP_SLO;
			return url;
		}

		public String getSpSLOLink() {
			//create url to Pingfederate SP Start SLO service
			String url = hostPFSp + Constants.START_SLO + "?" +
				Constants.TARGET_RESOURCE + "=" + hostAppSp + Constants.TARGET_SP_SLO;
			return url;
		}

		public String getIdpLoginPageUrl(String loginType) {
			//create url to login.aspx on the IdP part of the Sample Application
			String url = hostAppIdp + Constants.LOGIN_ASPX + "?" +
				Constants.LOGIN_TYPE + "=" + loginType;
			return url;
		}

		public String getSpLoginPageUrl(String loginType) {
			//create url to login.aspx on the SP part of the Sample Application
			String url = hostAppSp + Constants.LOGIN_ASPX + "?" +
				Constants.LOGIN_TYPE + "=" + loginType;
			return url;
		}

		public String getAccountLinkingSpPageUrl(String loginType) {
			//create url to accountlinking.aspx on the SP part of the Sample Application
			String url = hostAppSp + Constants.ACCOUNT_LINK_PAGE + "?" +
				Constants.LOGIN_TYPE + "=" + loginType;
			return url;
		}

		public String getIdpMainPageUrl() {
			//create url to mainpage.aspx on the IdP part of the Sample Application
			String url = hostAppIdp + Constants.MAIN_PAGE;
			return url;
		}

		public String getSpMainPageUrl() {
			//create url to mainpage.aspx on the SP part of the Sample Application
			String url = hostAppSp + Constants.MAIN_PAGE;
			return url;
		}

		public String getIdpErrorPageUrl() {
			//create url to error page on the IdP part of the Sample Application
			//this url allows to add some message to be shown on the error page
			String url = hostAppIdp + Constants.WEB_PREFIX + Constants.ERROR_PAGE;
			return url;
		}

		public String getSpErrorPageUrl() {
			//create url to error page on the SP part of the Sample Application
			//this url allows to add some message to be shown on the error page
			String url = hostAppSp + Constants.WEB_PREFIX + Constants.ERROR_PAGE;
			return url;
		}

		public String getResumePathUrl(String resumePath) {
			//create the full resumePath based on config properties
			String url = hostPF + resumePath;
			return url;
		}

		public String getSpDefederateUrl() {
			//create url to defederate service
			String url = hostPF + Constants.SP_DEFEDERATE;
			return url;
		}

		public String addParameter(String url, String name, String value) {
			String newURL = null;
			if (url.IndexOf('?') != -1) {
				newURL = url + '&' + name + '=' + value;
			}
			else {
				newURL = url + '?' + name + '=' + value;
			}
			return newURL;
		}

		private String addSSOParameters(String url, String forceAuthn, String isPassive) {
			//create string with parameters to be added to url
			String urlWithParams = url;
			if(forceAuthn != null && isPassive == null) {
				urlWithParams = addParameter(url, Constants.FORCE_AUTH_PARAM, forceAuthn);
			}
			if(forceAuthn == null && isPassive != null) {
				urlWithParams = addParameter(url, Constants.IS_PASSIVE_PARAM, isPassive);
			}
			if(forceAuthn != null && isPassive != null) {
				urlWithParams = addParameter(url, Constants.FORCE_AUTH_PARAM, forceAuthn);
				urlWithParams = addParameter(urlWithParams, Constants.IS_PASSIVE_PARAM, isPassive);
			}
			return urlWithParams;
		}
	}
}