/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;
using System.Collections;
using System.Configuration;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility;

namespace Com.PingIdentity.PingFederate.SampleApp.SpSample.web {

	/// <summary>
	/// Handles errors in SP Sample Application
	/// </summary>
	public class ErrorForm : Page {

		protected HtmlAnchor IdpHomeUrl;
		protected HtmlAnchor SpHomeUrl;
		protected Label      ErrorLabel;
	
		private void Page_Load(object sender, EventArgs e) {
			
			this.ErrorLabel.Text = Request.QueryString.Get(Constants.ERROR_MESSAGE);

			//get properties from Idp config file
			Hashtable idpConfigTable;
			if(Session[Constants.IDP_CONFIG_TABLE] == null) {
				idpConfigTable = GetSpConfigProps(this.Context);
				Session[Constants.IDP_CONFIG_TABLE] = idpConfigTable;
			}
			else {
				idpConfigTable = (Hashtable)Session[Constants.IDP_CONFIG_TABLE];
			}

			Links samlLinks = new Links(idpConfigTable, Request);
			this.IdpHomeUrl.HRef = samlLinks.getIdpHomeUrl();
			this.SpHomeUrl.HRef  = samlLinks.getSpHomeUrl();
		}

		/// <summary>
		/// Reads general configuration properties XML file for the
		/// SP Sample Application and converts it to Hastable object
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>Hashtable of user general configuration properties</returns>
		private Hashtable GetSpConfigProps(HttpContext context) {

			//read properties from Sp config XML file
			String spConfig  = ConfigurationSettings.AppSettings["config-file"];
			String configPath = context.Server.MapPath("config/" + spConfig);

			XmlUtility xmlUtility = new XmlUtility();
			Hashtable spConfigTable = xmlUtility.GetConfigProps(configPath);
			if(spConfigTable == null) {
				String errorMessage = "Error in " + spConfig;
				context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
			}
			return spConfigTable;
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}