/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;
using System.Collections;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Configuration;
using System.Web;
using System.Web.Security;
using Com.PingIdentity.Adapters.Pftoken.Core;
using Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility;

namespace Com.PingIdentity.PingFederate.SampleApp.SpSample.web {
	/// <summary>
	/// Shows user attributes and connections
	/// </summary>
	public class MainForm : Page {

		public const int MAX_LENGTH = 40;
		protected HtmlAnchor SpSSOUrl;
		protected HtmlAnchor SpSSOUrlWithForceAuthn;
		protected HtmlAnchor SpSSOUrlWithIsPassive;
		protected HtmlAnchor SpSLOUrl;

		private Links samlLinks;
	
		private void Page_Load(object sender, EventArgs e) {

			//prevent the output of aspx page from being cached by the browser
			Response.AddHeader("Cache-Control","no-cache");
			Response.AddHeader("Pragma","no-cache");

			//get properties from SP config file
			Hashtable spConfigTable = (Hashtable)Session[
				Constants.SP_CONFIG_TABLE];

			samlLinks = new Links(spConfigTable, Request);
			this.SpSLOUrl.HRef   = samlLinks.getSpStartSLOUrl();

			Hashtable userInfo = (Hashtable)Session[Constants.USER_INFO];
			if((userInfo == null) || (userInfo.Count <= 0)) {
				String resumePath = Request.QueryString.Get(Constants.RESUME_PATH);
				String loginPage = samlLinks.getSpLoginPageUrl(Constants.LOCAL_TYPE);
				if(resumePath != null) {
					loginPage = samlLinks.addParameter(loginPage, Constants.RESUME_PATH, resumePath);
				}
				Response.Redirect(loginPage);
			}
		}

		protected void ShowIdpConnections() 
		{
			//read properties from Idp connections XML file
			String spConnections  = ConfigurationSettings.AppSettings["connections-file"];
			String connectionsPath = Context.Server.MapPath("config/" + spConnections);
			if (!File.Exists(connectionsPath)) 
			{
				SSOEntity[] idpList = null;
				try 
				{
					SSODirectoryServiceService directoryService = new SSODirectoryServiceService();
					idpList = directoryService.getIDPList();
				}
				catch(Exception e) 
				{
					// Log exception
					HttpContext.Current.Trace.Warn(e.ToString());
					// Display alert
					Response.Write("<script language=\"javascript\">");
					Response.Write("alert('Note that a connection supporting SP initiated SSO should be configured before you can perform an SSO from the SP application. Please configure such a connection and ensure that the PingFederate server is running.')");
					Response.Write("</script>");
					return;
				}
				IEnumerator list = idpList.GetEnumerator();
				list.Reset();
				int i=0;
				while(list.MoveNext()) 
				{
					i++;
					SSOEntity idp = (SSOEntity)list.Current;
					String connectionName = (String)idp.entityId;
					String description = "Click on the image to initiate Single sign on";
					String spSSOUrl = null;
					
					spSSOUrl = samlLinks.getSpStartSSOUrl(connectionName,null,null);

					Response.Write("<table class=\"connection\">");
					Response.Write("<tr>");
					Response.Write("<td colspan=\"2\">");
					Response.Write("<div class=\"headersp\">");
					Response.Write("IdP Connection");

					Response.Write("<a href=\"javascript:openHelp('help.html?#sso');\">");
					Response.Write("<img src=\"images/HelpIcon.gif\" border=\"0\">");
					Response.Write("</a>");
					Response.Write("</div>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td colspan=\"2\" align=\"left\">");
					Response.Write("<span class=\"text\"><strong>Connection Id: </strong>");
					Response.Write(connectionName);
					Response.Write("</span></td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td colspan=\"2\">");
					Response.Write("<span class=\"text\">");
					Response.Write(description);
					Response.Write("</span>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td align=\"center\">");
					Response.Write("<input type=\"checkbox\" id=\"forceAuthn");
					Response.Write(i + 1);
					Response.Write("\" name=\"forceAuthn");
					Response.Write(i + 1);
					Response.Write("\"><span class=\"text\">");
					Response.Write("ForceAuthn");
					Response.Write("<a href=\"javascript:openHelp('help.html?#forceAuthn');\">");
					Response.Write("<img src=\"images/HelpIcon.gif\" border=\"0\">");
					Response.Write("</a>");
					Response.Write("</span>");
					Response.Write("</td>");
					Response.Write("<td align=\"center\">");
					Response.Write("<input type=\"checkbox\" id=\"isPassive");
					Response.Write(i + 1);
					Response.Write("\" name=\"isPassive");
					Response.Write(i + 1);
					Response.Write("\"><span class=\"text\">");
					Response.Write("IsPassive");
					Response.Write("<a href=\"javascript:openHelp('help.html?#isPassive');\">");
					Response.Write("<img src=\"images/HelpIcon.gif\" border=\"0\">");
					Response.Write("</a>");
					Response.Write("</span>");
					Response.Write("</td>");
					Response.Write("</tr>");

					Response.Write("<tr>");
					Response.Write("<td colspan=\"2\">");
					Response.Write("<a class=\"image\" href=\"#\">");
					Response.Write("<img id=\"federate");
					Response.Write(i + 1);
					Response.Write("\" name=\"federate");
					Response.Write(i + 1);
					Response.Write("\" border=\"0\"");
					Response.Write("src=\"images/Federate_Normal.gif\"");
					Response.Write("OnMouseOver=\"javascript:document.all['federate");
					Response.Write(i + 1);				
					Response.Write("'].src='images/Federate_Hover.gif';\"");
					Response.Write("OnMouseOut=\"javascript:document.all['federate");
					Response.Write(i + 1);				
					Response.Write("'].src='images/Federate_Normal.gif';\"");
					Response.Write("OnClick=\"javascript:initiateSSO('");
				
					Response.Write(spSSOUrl);
					Response.Write("','forceAuthn");
					Response.Write(i + 1);				
					Response.Write("','isPassive");
					Response.Write(i + 1);								
					Response.Write("')\">");
					Response.Write("</a>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("</table>");

				}
				// Check to see if any connections were obtained
				if(i ==0 ) 
				{
					Response.Write("<script language=\"javascript\">");
					Response.Write("alert('No IdP connections were found, please make sure PingFederate is running and there are connections configured.')");
					Response.Write("</script>");
				}
			}
			else 
			{
				//get properties from Sp connections file
				ArrayList spConnectionsList = (ArrayList)Session[Constants.SP_CONNECT_LIST];
				for(int i = 0; i < spConnectionsList.Count; i++) 
				{
					Hashtable connection  = (Hashtable)spConnectionsList[i];
					String connectionName = (String)connection[Constants.CONNECTION_NAME];
					String partnerIdpId   = (String)connection[Constants.CONNECTION_VALUE];
					String SpAdapterId    = (String)connection[Constants.SP_ADAPTER_ID];
					String IdpApp		  = (String)connection[Constants.IDP_APP];
					String description	  = (String)connection[Constants.DESCRIPTION];

					String spSSOUrl = samlLinks.getSpStartSSOUrl(partnerIdpId, SpAdapterId, null, null);

					String errorMessage = "Error+during+SSO+to+" + connectionName;
					spSSOUrl = samlLinks.addParameter(spSSOUrl, Constants.IN_ERROR_RESOURCE,
						samlLinks.getSpErrorPageUrl() + errorMessage);

					Response.Write("<table class=\"connection\">");
					Response.Write("<tr>");
					Response.Write("<td colspan=\"2\">");
					Response.Write("<div class=\"headersp\">");
					Response.Write("IdP Connection");
					Response.Write("<a href=\"javascript:openHelp('help.html?#sso');\">");
					Response.Write("<img src=\"images/HelpIcon.gif\" border=\"0\">");
					Response.Write("</a>");
					Response.Write("</div>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td colspan=\"2\" align=\"left\">");
					Response.Write("<span class=\"text\"><strong>Connection Id: </strong>");
					Response.Write(partnerIdpId);
					Response.Write("</span></td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td colspan=\"2\" align=\"left\">");
					Response.Write("<span class=\"text\"><strong>Description: </strong>");
					Response.Write(description);
					Response.Write("</strong></span>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td align=\"center\">");
					Response.Write("<input type=\"checkbox\" id=\"forceAuthn");
					Response.Write(i + 1);
					Response.Write("\" name=\"forceAuthn");
					Response.Write(i + 1);
					Response.Write("\"><span class=\"text\">");
					Response.Write("ForceAuthn");
					Response.Write("<a href=\"javascript:openHelp('help.html?#forceAuthn');\">");
					Response.Write("<img src=\"images/HelpIcon.gif\" border=\"0\">");
					Response.Write("</a>");
					Response.Write("</span>");
					Response.Write("</td>");
					Response.Write("<td align=\"center\">");
					Response.Write("<input type=\"checkbox\" id=\"isPassive");
					Response.Write(i + 1);
					Response.Write("\" name=\"isPassive");
					Response.Write(i + 1);
					Response.Write("\"><span class=\"text\">");
					Response.Write("IsPassive");
					Response.Write("<a href=\"javascript:openHelp('help.html?#isPassive');\">");
					Response.Write("<img src=\"images/HelpIcon.gif\" border=\"0\">");
					Response.Write("</a>");
					Response.Write("</span>");
					Response.Write("</td>");
					Response.Write("</tr>");

					Response.Write("<tr>");
					Response.Write("<td colspan=\"2\">");
					Response.Write("<a class=\"image\" href=\"#\">");
					Response.Write("<img id=\"federate");
					Response.Write(i + 1);
					Response.Write("\" name=\"federate");
					Response.Write(i + 1);
					Response.Write("\" border=\"0\"");
					Response.Write("src=\"images/Federate_Normal.gif\"");
					Response.Write("OnMouseOver=\"javascript:document.all['federate");
					Response.Write(i + 1);				
					Response.Write("'].src='images/Federate_Hover.gif';\"");
					Response.Write("OnMouseOut=\"javascript:document.all['federate");
					Response.Write(i + 1);				
					Response.Write("'].src='images/Federate_Normal.gif';\"");
					Response.Write("OnClick=\"javascript:initiateSSO('");
				
					Response.Write(spSSOUrl);
					Response.Write("','forceAuthn");
					Response.Write(i + 1);				
					Response.Write("','isPassive");
					Response.Write(i + 1);								
					Response.Write("')\">");
					Response.Write("</a>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("<tr>");
					Response.Write("<td colspan=\"2\">");
					Response.Write("<span class=\"text\">IdP App: <a class=\"text\" href=\"");
					Response.Write(IdpApp);				
					Response.Write("\">");
					Response.Write(IdpApp);				
					Response.Write("</a></span>");
					Response.Write("</td>");
					Response.Write("</tr>");
					Response.Write("</table>");
				}
			}
		}

		protected void ShowUserAttributes() 
		{

			Hashtable userInfo = (Hashtable)Session[Constants.USER_INFO];
			if(userInfo != null && userInfo.Count > 0) 
			{
				foreach(String key in userInfo.Keys) 
				{
					String infoName = key;
					String infoValue = (String)userInfo[key];

					// Handle Authn Context
					if(infoName == PftokenAgent.AUTHN_CTX_ATTRIBUTE_NAME) 
					{
						infoValue = Constants.DisplayAuthnContext(infoValue);
					}

					// Check if value is too long
					if(infoValue.Length > MAX_LENGTH) 
					{
						infoValue = infoValue.Substring(0,MAX_LENGTH);
						infoValue += "...";
					}

					Response.Write("<tr>");
					Response.Write("<td align=\"left\" width=\"50\">");
					Response.Write("<span class=\"text\">");
					Response.Write("<strong>");
					Response.Write(infoName);
					Response.Write("</strong>");
					Response.Write("</span>");
					Response.Write("</td>");
					Response.Write("<td align=\"left\">");
					Response.Write("<span class=\"text\">");
					Response.Write(infoValue);
					Response.Write("</span>");
					Response.Write("</td>");
					Response.Write("</tr>");

				}
			}
		}

		protected void ShowDefederateLink() {

			//get properties from SP config file
			Hashtable spConfigTable = (Hashtable)Session[Constants.SP_CONFIG_TABLE];

			//get account mode: true, if it's account linking
			String linkingMode = (String)spConfigTable[Constants.ACCOUNT_LINKING];
			if( String.Compare(linkingMode, Constants.TRUE_STRING_VALUE, true) != 0 ) {
				//It is not Account Linking mode. We don't have
				//to show link to defederate service
				return;
			}

			//Get user session list
			Hashtable userInfo = (Hashtable)Session[Constants.USER_INFO];

			if(userInfo != null && userInfo.Count > 0) {
				//Some sessions have been established via SSO.
				//We can activate defederate link.
				String spDefLink = samlLinks.getSpDefederateUrl();
				Response.Write("<br>");
				Response.Write("<table class=\"sub\">");
				Response.Write("<tr>");
				Response.Write("<td>");
				Response.Write("<div class=\"headersp\">");
				Response.Write("De-Federate");
				Response.Write("<a href=\"javascript:openHelp('help.html?#defederate');\">");
				Response.Write("<img src=\"images/HelpIcon.gif\" border=\"0\">");
				Response.Write("</a>");
				Response.Write("</div>");
				Response.Write("</td>");
				Response.Write("</tr>");
				Response.Write("<tr>");
				Response.Write("<td>");
				Response.Write("<span class=\"text\">Click on the link to disassociate account link with the IdP</span>");
				Response.Write("</td>");
				Response.Write("</tr>");
				Response.Write("<tr>");
				Response.Write("<td>");
				Response.Write("<a class=\"text\" href=\"");
				Response.Write(spDefLink);
				Response.Write("\">De-federate</a>");
				Response.Write("</td>");
				Response.Write("</tr>");
				Response.Write("</table>");

			}
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}